"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _embarkI18n = require("embark-i18n");

var _embarkjsEns = _interopRequireDefault(require("embarkjs-ens"));

const ENSFunctions = _embarkjsEns.default.ENSFunctions;

const namehash = require('eth-ens-namehash');

class ensAPI {
  constructor(embark, options) {
    this.embark = embark;
    this.ens = options.ens;
  }

  registerAPIs() {
    this.embark.registerAPICall('get', '/embark-api/ens/resolve', (req, res) => {
      var _context;

      ENSFunctions.resolveName(req.query.name, this.ens.ensContract, (0, _bind.default)(_context = this.ens.createResolverContract).call(_context, this.ens), (error, address) => {
        if (error) {
          return res.send({
            error: error.message || error
          });
        }

        res.send({
          address
        });
      }, namehash);
    });
    this.embark.registerAPICall('get', '/embark-api/ens/lookup', (req, res) => {
      var _context2;

      ENSFunctions.lookupAddress(req.query.address, this.ens.ensContract, namehash, (0, _bind.default)(_context2 = this.ens.createResolverContract).call(_context2, this.ens), (error, name) => {
        if (error) {
          return res.send({
            error: error.message || error
          });
        }

        res.send({
          name
        });
      });
    });
    this.embark.registerAPICall('post', '/embark-api/ens/register', (req, res) => {
      const {
        subdomain,
        address
      } = req.body;
      this.ens.ensRegisterSubdomain(subdomain, address, error => {
        if (error) {
          return res.send({
            error: error.message || error
          });
        }

        res.send({
          name: `${req.body.subdomain}.${this.embark.config.namesystemConfig.register.rootDomain}`,
          address: req.body.address
        });
      });
    });
  }

  registerConsoleCommands() {
    this.embark.registerConsoleCommand({
      usage: 'resolve [name]',
      description: (0, _embarkI18n.__)('Resolves an ENS name'),
      matches: cmd => cmd.split(' ')[0] === 'resolve',
      process: (cmd, cb) => {
        var _context3;

        const [_cmdName, name] = cmd.split(' ');
        ENSFunctions.resolveName(name, this.ens.ensContract, (0, _bind.default)(_context3 = this.ens.createResolverContract).call(_context3, this.ens), cb, namehash);
      }
    });
    this.embark.registerConsoleCommand({
      usage: 'lookup [address]',
      description: (0, _embarkI18n.__)('Lookup an ENS address'),
      matches: cmd => cmd.split(' ')[0] === 'lookup',
      process: (cmd, cb) => {
        var _context4;

        const [_cmdName, address] = cmd.split(' ');
        ENSFunctions.lookupAddress(address, this.ens.ensContract, namehash, (0, _bind.default)(_context4 = this.ens.createResolverContract).call(_context4, this.ens), cb);
      }
    });
    this.embark.registerConsoleCommand({
      usage: 'registerSubDomain [subDomain] [address]',
      description: (0, _embarkI18n.__)('Register an ENS sub-domain'),
      matches: cmd => cmd.split(' ')[0] === 'registerSubDomain',
      process: (cmd, cb) => {
        const [_cmdName, name, address] = cmd.split(' ');
        this.embark.events.request("blockchain:defaultAccount:get", (_err, defaultAccount) => {
          this.ens.safeRegisterSubDomain(name, address, defaultAccount, cb);
        });
      }
    });
  }

}

module.exports = ensAPI;
//# sourceMappingURL=api.js.map